import { ChangeEvent, FormEvent, useState } from "react";
import {
  Alert,
  Avatar,
  Box,
  Button,
  Container,
  Grid,
  Link,
  Typography,
} from "@mui/material";
import LockOutlinedIcon from "@mui/icons-material/LockOutlined";
import { useAppDispatch, useAppSelector } from "../../store/hooks.ts";
import { useNavigate } from "react-router-dom";
import { registerUser } from "../../features/usersSlice.ts";
import FormElement from "../../components/UI/Form/FormElement/FormElement.tsx";

interface IRegisterState {
  username: string;
  password: string;
  displayName: string;
}
const Register = () => {
  const { registerError } = useAppSelector((state) => state.user);
  const dispatch = useAppDispatch();
  const navigate = useNavigate();

  const [state, setState] = useState<IRegisterState>({
    username: "",
    password: "",
    displayName: "",
  });

  const inputChangeHandler = (e: ChangeEvent<HTMLInputElement>) => {
    const { name, value } = e.target;

    setState((prevState) => ({ ...prevState, [name]: value }));
  };

  const submitFormHandler = (e: FormEvent<HTMLFormElement>) => {
    e.preventDefault();

    dispatch(registerUser({ ...state }))
      .unwrap()
      .then(() => {
        navigate("/");
      });
  };

  const getErrorsBy = (name: string) => {
    if (Array.isArray(registerError)) {
      const currentError = registerError.find(({ type }) => type === name);

      return currentError?.messages.join(",");
    }
  };

  return (
    <Container component="main" maxWidth="xs">
      <Box
        sx={{
          marginTop: 8,
          display: "flex",
          flexDirection: "column",
          alignItems: "center",
        }}
      >
        <Avatar sx={{ m: 1, bgcolor: "secondary.main" }}>
          <LockOutlinedIcon />
        </Avatar>

        <Typography component="h1" variant="h5">
          Sign up
        </Typography>

        {registerError && !Array.isArray(registerError) ? (
          <Alert severity="error">{registerError}</Alert>
        ) : null}

        <Box
          component="form"
          onSubmit={submitFormHandler}
          noValidate
          sx={{ mt: 1 }}
        >
          <FormElement
            label="Your name"
            value={state.displayName}
            onChange={inputChangeHandler}
            name="displayName"
          />

          <FormElement
            label="Login"
            value={state.username}
            onChange={inputChangeHandler}
            name="username"
            error={getErrorsBy("username")}
          />

          <FormElement
            label="Password"
            value={state.password}
            onChange={inputChangeHandler}
            name="password"
            error={getErrorsBy("password")}
          />

          <Button
            type="submit"
            fullWidth
            variant="contained"
            sx={{ mt: 3, mb: 2 }}
          >
            Sign Up
          </Button>

          <Grid container>
            <Grid item>
              <Link href="#" variant="body2">
                {"Have an account? Sign In"}
              </Link>
            </Grid>
          </Grid>
        </Box>
      </Box>
    </Container>
  );
};

export default Register;
