import { IUser } from "../interfaces/IUser.ts";
import { createAsyncThunk, createSlice } from "@reduxjs/toolkit";
import axiosApi from "../axiosApi.ts";
import { AxiosError, isAxiosError } from "axios";

type userResponseValidateError = {
  type: string;
  messages: string[];
}[];

interface IUserState {
  user: IUser | null;
  loading: boolean;
  registerError: string | null | userResponseValidateError;
  loginError: null | string;
}

type userRequest = {
  username: string;
  displayName?: string;
  password: string;
};

type userResponseError = {
  error: { message: string };
};

export const registerUser = createAsyncThunk<
  IUser,
  userRequest,
  { rejectValue: userResponseError | userResponseValidateError }
>("auth/register", async (userData: userRequest, { rejectWithValue }) => {
  try {
    const response = await axiosApi.post<IUser>("/auth/register", userData);
    return response.data;
  } catch (e) {
    if (isAxiosError(e)) {
      const error: AxiosError<userResponseError> = e;
      return rejectWithValue(
        error.response?.data || { error: { message: "An error occurred" } },
      );
    }
    throw e;
  }
});

export const loginUser = createAsyncThunk<
  IUser,
  userRequest,
  { rejectValue: string }
>("auth/login", async (userData: userRequest, { rejectWithValue }) => {
  try {
    const response = await axiosApi.post<IUser>("/auth/sign-in", userData);
    return response.data;
  } catch (e) {
    if (isAxiosError(e)) {
      const error: AxiosError<userResponseError> = e;
      return rejectWithValue(
        error.response?.data.error.message || "An error occured",
      );
    }

    throw e;
  }
});

export const logoutUser = createAsyncThunk(
  "auth/logout",
  async (_, { rejectWithValue }) => {
    try {
      const response = await axiosApi.delete("/auth/logout");
      return response.data;
    } catch (err) {
      if (isAxiosError(err)) {
        const error: AxiosError<userResponseError> = err;
        return rejectWithValue(
          error.response?.data.error.message || "Internet connection error",
        );
      }
      throw err;
    }
  },
);

const initialState: IUserState = {
  user: null,
  loading: false,
  registerError: null,
  loginError: null,
};

const userSlice = createSlice({
  name: "user",
  initialState,
  reducers: {},
  extraReducers: (builder) => {
    builder
      .addCase(registerUser.pending, (state) => {
        state.loading = true;
        state.registerError = null;
      })
      .addCase(registerUser.fulfilled, (state, action) => {
        state.loading = false;
        state.user = action.payload ?? null;
      })
      .addCase(registerUser.rejected, (state, action) => {
        state.loading = false;

        if (Array.isArray(action.payload)) {
          state.registerError = action.payload;
        } else {
          state.registerError =
            action.payload?.error.message ?? "Error occured";
        }
      })
      .addCase(loginUser.pending, (state) => {
        state.loading = true;
        state.loginError = null;
      })
      .addCase(loginUser.fulfilled, (state, action) => {
        state.loading = false;
        state.user = action.payload;
      })
      .addCase(loginUser.rejected, (state, action) => {
        state.loading = false;
        state.loginError = action.payload || null;
      })
      .addCase(logoutUser.fulfilled, () => {
        return initialState;
      });
  },
});

export default userSlice.reducer;
